/*
 * \file: table.cpp
 */

#include "table.h"
#include "disc.h"
#include "memtrace.h"

Table::Table(int nx, int ny) :vege(false), t(nx, std::vector<Disc*>(ny)) {
    int k = 0;
    /// Kitöltjük az irányok tábláját
    for (int i = -1; i <= 1; i++) {
        for (int j = -1; j <= 1; j++) {
            if (i != 0 || j != 0) {
                dirs[k].dx = i;
                dirs[k].dy = j;
                k++;
            }
        }
    }
}

Table::~Table() {
    for (size_t i = 0; i < t.size(); i++)
        for (size_t j = 0; j < t[0].size(); j++)
            delete t[i][j];
}

/// Adott irányban levő szomszéd lekérdezése
/// @param x - X pozíció
/// @param y - Y pozíció
/// @param dir - irány [0..MaxDir]
/// @return szomszéd pointere, vagy null
Disc* Table::getNext(int x, int y, int dir) const {
    int nx = x + dirs[dir].dx;
    int ny = y + dirs[dir].dy;
    if (validx(nx) && validy(ny))
        return t[nx][ny];
    else
        return NULL;
}

/// Szomszédok lekérdezése.
/// @param x - X pozíció
/// @param y - Y pozíció
/// @return szomszédok map-je (irány, Disc*)
Table::neighbors_type Table::getNeighbors(int x, int y) {
    neighbors_type nb;
    for (int i = 0; i < MaxDir; i++) {
        int nx = x + dirs[i].dx;
        int ny = y + dirs[i].dy;
        if (validx(nx) && validy(ny) && t[nx][ny] != NULL)
            nb[i] = t[nx][ny];
    }
    return nb;
}

/// Adott hely tesztelése
/// @param x - X pozíció
/// @param y - Y pozíció
/// @return true, ha szabad a hely
bool Table::probe(int x, int y) {
    if (validx(x) && validy(y) && t[x][y] == NULL)
        return true;
    return false;
}

/// Korong felrakása a táblára
/// @param d - korong pointere
void Table::setDisc(Disc* d) {
    Position pos = d->getPosition();
    t[pos.getx()][pos.gety()] = d;
    // Megkérjük, hogy ellenőrizze, hogy van-e már elegendő korong
    vege = d->checkAll();
}

/// Tabla kirajzolása
void Table::list() const {
    std::cout << ' ';
    for (size_t j = 0; j < t[0].size(); j++)
        std::cout << "|" << j;
    std::cout << "|\n";
    for (size_t i = 0; i < t.size(); i++) {
        std::cout << i;
        for (size_t j = 0; j < t[0].size(); j++) {
            Disc* dp = t[i][j];
            if (dp == NULL)
                std::cout << "|.";
            else if (dp->getColor() == color_type(0))
                std::cout << "|O";
            else if (dp->getColor() == color_type(1))
                std::cout << "|X";
            else
                std::cout << "|*";
        }
        std::cout << "|\n";
    }
}
