/**
 * \file obj.cpp
 *    Obj és URES objektumok a labirintus programhoz.
 * $Id: obj.cpp 499 2015-05-05 10:53:36Z szebi $
 */

#include <stdexcept>
#include "memtrace.h"
#include "obj.h"
#include "cella.h"


/*
 * Nez tagfüggvény.
 * Visszaadja a paraméterként kapott cella adott
 * szomszédjának típusát.
 */
Obj* Obj::Nez(int i, Cella &c, bool b) {
    if (Cella *p = c.GetSz(i)) {                // ha van szomszéd cella
      if (b || p->GetNyom(this) < 0)            // de nincs nyoma, vagy visszalép
        return(p->GetObj());                    // akkor az cellán álló objektum pointerét adja
    }
    return(NULL);                               // egyébként NULL-t
}

/*
 * Obj Lepj tagfügvény.
 * A megadott ianyba lepteti az objekumot.
 */
void Obj::Lepj(int i, Cella &c, bool b) {
    if (Cella *p = c.GetSz(i)) {                // szomszéd cella
        p->SetObj(c.GetObj());                  // beírja saját magát
        if (!b)                                 // ha nem visszalépés
          p->SetNyom(ir^2);                     // erről jött (szemben levő irányok a 2-es bitben különböznek)
        c.SetObj(&URES);                        // törli saját magát
    }
}

/*
 * Obj Lep tagfügvénye.
 * Bejárásnak megfeleően lép
 */
void Obj::Lep(Cella &c) {
    const Cella::irany F = Cella::F;            // így rövidebben tudunk rá hivatkozni
    if (c.GetNyom(this) < 0)                    // ha nem volt nyoma, akkor ez a kiindulási pont
      c.SetNyom(F+1);                           // így nem tud majd innen visszalépni (F+1)
    if (++ir <= F) {                            // ha van még bejáratlan irány
      if (Obj *p = Nez(ir, c)) {                // és ha van szomszéd
        if (Ereje() > p->Ereje()) {             // és Õ az erősebb
          (*p)(this);                           // meghívjuk a függvényhívás operátorát
          Lepj(ir, c);                          // rálépünk (eltapossuk)
          ir = -1;                              // most léptünk be: kezdő irány
        }
      }
    } else {
      if ((ir = c.GetNyom(this)) > F) {         // kezdő pozícióból nem tud visszalépni
          throw std::logic_error("Kezdo pozicioba jutott");
      } else {                                  // visszalépés
        if (Obj *p = Nez(ir, c, true)) {        // lehet, hogy közben valaki odalépett, ezt ellenőrizni kell
          if (Ereje() > p->Ereje()) {           // Õ az erősebb
            (*p)(this);                         // meghívjuk a függvényhívás operátorát
            Lepj(ir, c, true);                  // visszalépünk és eltapossuk
            ir ^= 2;                            // erre ment be (trükk: szemben levő irányok a 2-es bitben különböznek)
          }
        }
      }
    }
}

Ures URES;                                      // Üres objektum létrehozása egyetlen példányban

